package com.vanheusden.pfa;

final class Board {
	private final ChessObject [][] board = new ChessObject[8][8];

	public Board() {
	}

	public Board(final ChessObject [] white, final ChessObject [] black) {
		/* put objects on board */
		for(final ChessObject cur : white)
			board[cur.getY()][cur.getX()] = cur;

		for(final ChessObject cur : black)
			board[cur.getY()][cur.getX()] = cur;
	}

	public ChessObject [][] getAll() {
		return board;
	}

	public ChessObject getAt(final int x, final int y) {
		return board[y][x];
	}

	public void replace(final ChessObject object) {
		final int x = object.getX();
		final int y = object.getY();

		assert getAt(x, y) != null;

		board[y][x] = object;
	}

	public void set(final ChessObject object) {
		final int x = object.getX();
		final int y = object.getY();

		assert getAt(x, y) == null;

		board[y][x] = object;
	}

	public void removeForEnPassant(final int x, final int y) {
		assert y == 4 || y == 3;
		assert board[y][x] != null;

		board[y][x] = null;
	}

	public void moveObject(final int fx, final int fy, final int tx, final int ty) {
		assert fx != tx || fy != ty;
		assert board[fy][fx] != null;

		board[ty][tx] = board[fy][fx];
		board[fy][fx] = null;
	}

	public void moveObject(final Move move) {
		moveObject(move.getFX(), move.getFY(), move.getTX(), move.getTY());
	}
}
