package com.vanheusden.pfa;

import java.io.*;
import java.sql.*;
import java.util.*;

public class BookWDLStatic extends Book {
	public BookWDLStatic(final String url) throws Exception {
		super(url);

		if (table == null)
			table = "wdl_static";
	}

	public MoveResult search(final Scene s, final Move lastMove, final PlayerColor who) throws Exception {
		MoveResult selectedMove = null;

		try {
			final String fen_before = Statics.toFENString(s.getBoard(), who, lastMove, 0, true);

			String query = "select fen_after, move, plies, -eval AS eval, pv, ((win + 1.9208) / (win + loose) - 1.96 * SQRT((win * loose) / (win + loose) + 0.9604) / (win + loose)) / (1 + 3.8416 / (win + loose)) AS ci_lower_bound from " + table + ", entries where fen=fen_after and fen_before='" + fen_before + "' and win + loose >= " + min_hits + " order by ci_lower_bound desc limit 1";

			ResultSet getRS = get.executeQuery(query.toString());

			if (!getRS.next())
				return null;

			String fen_after = getRS.getString("fen_after");

			List<Move> moveList = s.getMoveList(who);
			if (moveList.isEmpty()) {
				Statics.log("BookWDLStatic (" + name + "): movelist is empty");
				return null;
			}

			for(Move currentMove : moveList) {
				Scene sa = s.Move(currentMove);
				final Board b = sa.getBoard();
				final String fen = Statics.toFENString(b, Statics.opponentColor(who), currentMove, 0, true);

				if (fen.equals(fen_after)) {
					selectedMove = new MoveResult(getRS.getInt("eval"), getRS.getInt("plies"), getRS.getString("pv"), currentMove, getName());
					break;
				}
			}

			if (selectedMove != null)
				Statics.log("BookWDLStatic (" + name + "): found " + selectedMove.getMove());
		}
		catch(Exception e) {
			Statics.logException("BookWDLStatic (" + name + ")", e);
			throw e;
		}

		return selectedMove;
	}
}
