package com.vanheusden.pfa;

import java.io.*;
import java.util.*;
import java.util.concurrent.*;
import java.util.concurrent.atomic.*;
import java.util.concurrent.locks.*;

class Brain extends Thread {
	final Random r = new Random(System.currentTimeMillis());
	final boolean fairSemaphore = false;
	IO io;
	//
	final static int value_infinite = 10000, value_checkmate = 10000;
	final int aspirationWindowSize = 100;
	//
	int endDepth = 4, maxExtension = -5;
	boolean allowRandom = false;
	final long cleanupTimeFrame = 500; // ms to clean-up/output move
	//
	final GcThread gcThread = new GcThread();
	final Dummy myMonitorObject = new Dummy();
	final Thread gcThreadT = new Thread(gcThread, "GC Thread");
	//
	Semaphore [] threadCountLock;
	int [] runningThreadsCount, maxThreadsPerSlot;
	int currentMaxNProcs = 1, maxNProcs = 1;
	int threadsPerSlot = 8;
	int numberOfThreadSlots, extraThreads;
	final AtomicBoolean toDetected = new AtomicBoolean(false);
	int currentSearchDepth = 0;
	//
	long startedAt = 0;
	private AtomicLong [] nNodesProcessed, nQueiesceNodes;
	private AtomicLong [] nExtensions;
	private AtomicLong [] nCutOffs;
	private AtomicLong nThreadsCreated;
	//
	// parameters while searching
	long finishBefore;
	int moveNr = -1;
	//
	final TranspositionTable transTab;
	private AtomicLong [] nTransTab;
	final boolean ttEnabled;
	//
	final Zobrist z;
	//
	List<Book> obl = null;
	List<BookThread> bookThreads = null;
	//
	final boolean allowQueiescence;
        //
        int mateAtMin = Integer.MAX_VALUE, mateAtMax = Integer.MIN_VALUE;
        final Semaphore mateAtLock;
	// global stop
	AtomicBoolean stopAll = new AtomicBoolean(false);


	// for testcases
	long getTPTHits() {
		return sum(nTransTab);
	}

	class MSCvictimType implements Comparator<Move> {
		final Board brd;

		MSCvictimType(final Board brd) {
			this.brd = brd;
		}

		@Override
		// "a negative integer, zero, or a positive integer as the first argument is
		// less than, equal to, or greater than the second"
		// http://docs.oracle.com/javase/7/docs/api/java/util/Comparator.html
		// The goal here is to have the biggest victim with the smallest attacker.
		// victim: a > b
		// attacker: a < b (or: 7 - a > 7 - b)
		public int compare(Move a, Move b) {
			final ChessObject ao = brd.getAt(a.getFX(), a.getFY()); // from
			final ChessObject ad = brd.getAt(a.getTX(), a.getTY()); // to
			final ChessObject bo = brd.getAt(b.getFX(), b.getFY());
			final ChessObject bd = brd.getAt(b.getTX(), b.getTY());

			final int attackerValueA = ao.getType().getValue();
			final int victimValueA = ad != null ? ad.getType().getValue() : 0;

			final int attackerValueB = bo.getType().getValue();
			final int victimValueB = bd != null ? bd.getType().getValue() : 0;

			final int A = (victimValueA << 3) | (7 - attackerValueA);
			final int B = (victimValueB << 3) | (7 - attackerValueB);

			assert A >= 0;
			assert A < 64;
			assert B >= 0;
			assert B < 64;

			return B - A;
		}
	}

	class MSCshallowEval implements Comparator<Move> {
		PlayerColor side;

		MSCshallowEval(final PlayerColor side) {
			this.side = side;
		}

		@Override
		public int compare(Move a, Move b) {
			final int av = calcShannonValue(a.getScene(), side);
			final int bv = calcShannonValue(b.getScene(), side);

			return bv - av; // bv-av to get reverse order!! (big to small)
		}
	}

	long sum(AtomicLong [] array) {
		long total = 0;

		for(int nr=0; nr<array.length; nr++)
			total += array[nr].get();

		return total;
	}

	class RunStats implements Runnable {
		long prevTs = -1, prevNodeCount = -1;
		double maxNPS = -1.0;

		public void run() {
			try {
				try {
					Thread.sleep(1000);

					final Thread me = Thread.currentThread();

					// count running threads
					final ThreadGroup tg = me.getThreadGroup();

					final Thread [] list = new Thread[1024];

					for(;;) {
						final int n = tg.enumerate(list);

						final long gctId = gcThreadT.getId();
						final long myId = me.getId();

						int cntRunnable = 0, cntBlocked = 0;
						for(int i=0; i<n; i++) {
							final Thread.State st = list[i].getState();

							if (st == State.RUNNABLE) {
								final long curId = list[i].getId();
								// System.out.println("" + curId + " " + gctId + " " + myId + " " + list[i].getName());

								if (curId != gctId && curId != myId) {
									cntRunnable++;

	/*
									StackTraceElement [] sts = list[i].getStackTrace();
									StackTraceElement ste = sts[0];

									io.progressCallbackDebug(" " + list[i].getName() + "] " + ste.getClassName() + ", " + ste.getFileName() + ", " + ste.getLineNumber() + ", " + ste.getMethodName() + ", " + (ste.isNativeMethod() ?  "is" : "NOT a") + " native method");
	*/
								}
							}
							else if (st != State.NEW && st != State.TERMINATED) {
								final long curId = list[i].getId();

								if (curId != gctId && curId != myId) {
									cntBlocked++;
/*
									final StackTraceElement [] sts = list[i].getStackTrace();
									for(int nr=0; nr<Math.min(sts.length, 3); nr++) {
										StackTraceElement ste = sts[nr];

										io.progressCallbackDebug(" " + nr + "] " + list[i].getName() + "] " + ste.getClassName() + ", " + ste.getFileName() + ", " + ste.getLineNumber() + ", " + ste.getMethodName() + ", " + (ste.isNativeMethod() ?  "is" : "NOT a") + " native method");
									}
*/
								}
							}
						}

						String out = moveNr >= 0 ? "" + moveNr + "] " : "";

						out += "runnable: " + cntRunnable + ", blocked: " + cntBlocked;

						// calc thread slot usage
						int slotsUsage = 0, slotsInUse = 0;
						for(int index=0; index<numberOfThreadSlots; index++)
						{
							threadCountLock[index].acquireUninterruptibly();
							slotsUsage += runningThreadsCount[index];
							if (runningThreadsCount[index] != 0)
								slotsInUse++;
							threadCountLock[index].release();
						}
						out += ", threads: " + slotsUsage + ", slots: " + slotsInUse + "/" + numberOfThreadSlots;

						io.progressThreadStats(cntRunnable, cntBlocked, slotsUsage, slotsInUse, nThreadsCreated.get());

						final long curNodesProcessed = sum(nNodesProcessed);
						out += ", node count: " + curNodesProcessed;
						io.progressCallbackNNodes(curNodesProcessed);

						// determine nodes/s
						final long nowTs = System.currentTimeMillis();

						if (prevTs != -1) {
							final double curNPS = (curNodesProcessed - prevNodeCount) * 1000.0 / (double)(nowTs - prevTs);

							if (curNPS > maxNPS)
								maxNPS = curNPS;

							out += ", nodes/s: " + String.format("%.2f", curNPS);

							io.progressCallbackNPS((int)curNPS);
						}

						prevTs = nowTs;
						prevNodeCount = curNodesProcessed;

						out += ", extensions: " + sum(nExtensions) + ", tpt hits: " + sum(nTransTab) + ", cut offs: " + sum(nCutOffs) + ", tcr: " + nThreadsCreated.get() +", qn: " + sum(nQueiesceNodes);

						io.processChessStats(sum(nExtensions), sum(nTransTab), sum(nCutOffs), sum(nQueiesceNodes));

						if (finishBefore > 0)
							out += ", tl: " + (finishBefore - System.currentTimeMillis()) / 1000.0;

						io.progressCallbackDebug(out);

						io.progressTTLockDistribution(transTab.getLockingStats()[1]);

						Thread.sleep(3000);
					}
				}
				catch(InterruptedException bla) {
					if (maxNPS > 0.0)
						io.progressCallbackDebug("Peak NPS: " + maxNPS);
				}
			}
			catch(IOException ie) {
				io.exception(ie);
			}
		}
	}

	void setAllowRandom(final boolean allowRandom) {
		this.allowRandom = allowRandom;
	}

	void setNCores(final int cores) throws IOException {
		maxNProcs = cores;

		if (maxNProcs == 0)
			currentMaxNProcs = Runtime.getRuntime().availableProcessors();
		else
			currentMaxNProcs = Math.min(maxNProcs, Runtime.getRuntime().availableProcessors());

		extraThreads = currentMaxNProcs % threadsPerSlot;
		numberOfThreadSlots = currentMaxNProcs / threadsPerSlot + (extraThreads > 0 ? 1 : 0);

		threadCountLock = new Semaphore[numberOfThreadSlots];
		runningThreadsCount = new int[numberOfThreadSlots];
		maxThreadsPerSlot = new int[numberOfThreadSlots];

		for(int index=0; index<numberOfThreadSlots; index++) {
			if (Statics.getDebug())
				threadCountLock[index] = new MySem(1, fairSemaphore, "threadCount " + index);
			else
				threadCountLock[index] = new Semaphore(1, fairSemaphore);
		}

		int dummyToGive = currentMaxNProcs;
		for(int index=0; index<numberOfThreadSlots; index++) {
			runningThreadsCount[index] = 0;

			int n = Math.min(dummyToGive, threadsPerSlot);
			maxThreadsPerSlot[index] = n;
			dummyToGive -= n;
		}

		if (extraThreads > 0 && numberOfThreadSlots > 1) {
			int fullNumberSlot = numberOfThreadSlots - 2;
			int lessNumberSlot = numberOfThreadSlots - 1;

			int dummy = maxThreadsPerSlot[fullNumberSlot] + maxThreadsPerSlot[lessNumberSlot];

			maxThreadsPerSlot[fullNumberSlot] = dummy / 2;
			maxThreadsPerSlot[lessNumberSlot] = dummy - maxThreadsPerSlot[fullNumberSlot];
		}

		setupCounters();

		io.progressCallbackDebug("cores: " + currentMaxNProcs);
	}

	void setupCounters() {
		nNodesProcessed = new AtomicLong[currentMaxNProcs];
		nQueiesceNodes = new AtomicLong[currentMaxNProcs];
		nExtensions = new AtomicLong[currentMaxNProcs];
		nTransTab = new AtomicLong[currentMaxNProcs];
		nCutOffs = new AtomicLong[currentMaxNProcs];

		for(int index=0; index<currentMaxNProcs; index++) {
			nNodesProcessed[index] = new AtomicLong(0);
			nQueiesceNodes[index] = new AtomicLong(0);
			nExtensions[index] = new AtomicLong(0);
			nTransTab[index] = new AtomicLong(0);
			nCutOffs[index] = new AtomicLong(0);
		}

		nThreadsCreated = new AtomicLong(0);
	}

	class Dummy {
	}

	class GcThread implements Runnable {
		public void waitForPoll() throws InterruptedException {
			synchronized(myMonitorObject) {
				myMonitorObject.wait();
			}
		}

		public void poll() {
			synchronized(myMonitorObject){
				myMonitorObject.notifyAll();
			}
		}

		public void run() {
			try {
				for(;;) {
					System.runFinalization();
					System.gc();

					waitForPoll();
				}
			}
			catch(InterruptedException ie) {
			}
		}
	}

	long getHash(Board b, PlayerColor pc) {
		return z.calc(b, pc);
	}

	// needed by testcases
	protected TranspositionTable getTpt() {
		return transTab;
	}

	Brain(String transTabFile, int transTabSize, final int maxNProc, final int threadsPerSlot, final List<Book> books, final boolean allowQueiescence) throws Exception {
		this(transTabFile, transTabSize, -1, maxNProc, threadsPerSlot, books, allowQueiescence);
	}

	Brain(String transTabFile, int transTabSize, int TPTSyncInterval, final int maxNProc, final int threadsPerSlot, final List<Book> books, final boolean allowQueiescence) throws Exception {
		io = new IO_logonly();

		r.setSeed(System.currentTimeMillis());

		gcThreadT.setPriority(Thread.MIN_PRIORITY);
		gcThreadT.setDaemon(true);
		gcThreadT.start();

		this.allowQueiescence = allowQueiescence;

		transTab = new TranspositionTable(transTabFile, transTabSize, io);
		z = transTab.getZobrist();

		ttEnabled = transTabSize > 0;

		if (Statics.getDebug())
			mateAtLock = new MySem(1, fairSemaphore, "matAtLock");
		else
			mateAtLock = new Semaphore(1, fairSemaphore);

		this.threadsPerSlot = threadsPerSlot;

		setNCores(maxNProc);

		obl = books;
		setupBookThreads(obl);
	}

	void setIO(final IO io) {
		this.io = io;
	}

	void destructor() throws IOException {
		if (gcThreadT != null)
			gcThreadT.interrupt();

		if (transTab != null)
			transTab.destructor();

		if (mateAtLock instanceof MySem) {
			((MySem)mateAtLock).destructor();

			for(int index=0; index<numberOfThreadSlots; index++)
				((MySem)threadCountLock[index]).destructor();
		}
	}

	int countNLeftBehindPawns(final PlayerColor myColor, final ChessObject [] myObjects, final List<Move> hisMoves) { // backward pawn
		final int vulnerableLine1 = myColor == PlayerColor.WHITE ? 2 : 5;
		final int vulnerableLine2 = myColor == PlayerColor.WHITE ? 3 : 4;

		int n = 0;

		for(int index=0; index<myObjects.length; index++) {
			final ChessObject curObject = myObjects[index];

			if (!(curObject instanceof ChessObjectPawn))
				continue;

			// FIXME: not enough, need to check if moving to next row makes it vulnerable
			if (curObject.isFirstMove()) {
				// it does not matter if these values are invalid (-1/8) as they
				// will not be used to address locations in arrays
				final int x = curObject.getX();
				final int lx = x - 1;
				final int rx = x + 1;

				int nHisMoves = hisMoves.size();
				for(int hmIndex=0; hmIndex<nHisMoves; hmIndex++) {
					Move curMove = hisMoves.get(hmIndex);

					if (curMove.getTY() == vulnerableLine2 && curMove.getTX() == x) {
						n++;
						break;
					}

					if (curMove.getTY() == vulnerableLine1 && (curMove.getTX() == lx || curMove.getTX() == rx)) {
						n++;
						break;
					}
				}
			}
		}

		assert n <= 8;

		return n;
	}

	int countTotalObjectsValue(final ChessObject [] objects) {
		final PlayerColor mc = objects[0].getColor();
		int total = 0, bCnt = 0, pCnt = 0;

		for(int index=0; index<objects.length; index++) {
			final ChessObject cur = objects[index];
			final ChessObjectType cot = cur.getType();

			// maximum: 4075
			total += cot.getEvalVal();

			// range is -50...50 so in this case: 12
			total += ALG_position.evalPositions(cur, mc, false) / 4;

			switch(cot) {
				case PAWN:
					pCnt++;
					break;
				case BISHOP:
					bCnt++;
					break;
			}
		}

		// http://home.comcast.net/~danheisman/Articles/evaluation_of_material_imbalance.htm
		if (bCnt >= 2)
			total += 25;

		if (pCnt == 0)
			total -= 10; // FIXME guestimate by fvh

		// total maximum: 4075 (material) + 16 * 12 + 25 = 4292

		return total;
	}

	int [] getNumberOfPawnsPerFile(final ChessObject [] objects) {
		final int [] countsPerFile = new int[8];

		for(int index=0; index<objects.length; index++) {
			final ChessObject cur = objects[index];

			if (cur instanceof ChessObjectPawn)
				countsPerFile[cur.getX()]++;
		}

		return countsPerFile;
	}

	int countNIsolatedPawns(final int [] countsPerFile) {
		int n = 0;

		for(int x=0; x<8; x++) {
			if (countsPerFile[x] > 0) {
				if (x == 0) {
					if (countsPerFile[x + 1] == 0)
						n++;
				}
				else if (x == 7) {
					if (countsPerFile[x - 1] == 0)
						n++;
				}
				else {
					if (countsPerFile[x - 1] == 0 && countsPerFile[x + 1] == 0)
						n++;
				}
			}
		}

		assert n <= 8;

		return n;
	}

	int countNDoublePawns(final int [] countsPerFile) {
		int n = 0;

		for(int x=0; x<8; x++) {
			if (countsPerFile[x] >= 2)
				n += countsPerFile[x];
		}

		assert n <= 8;

		return n;
	}

	int calcMobility(final Scene scene, final PlayerColor who) {
		Board b = scene.getBoard();
		final List<Move> moves = scene.getMoveList(who);

		int count = 0;

		final int nMoves = moves.size();
		for(int index=0; index<nMoves; index++) {
			final Move current = moves.get(index);

			final ChessObject obj = b.getAt(current.getFX(), current.getFY());
			final ChessObjectType ot = obj.getType();

			if (ot != ChessObjectType.KING && ot != ChessObjectType.PAWN)
				count++;
		}

		return count;
	}

	int calcShannonValue(final Scene scene, final PlayerColor me) {
		final PlayerColor opponent = Statics.opponentColor(me);

		// these two must be at the top so that whe have all ChessObjects up-to-date (due to implicit movelist creation)
		final List<Move> myMoves = scene.getMoveList(me);
		final List<Move> hisMoves = scene.getMoveList(opponent);

		final int nMeMoves = myMoves.size();
		final int nOpponentMoves = hisMoves.size();

		final ChessObject [] myObjects = scene.getObjects(me);
		final ChessObject [] hisObjects = scene.getObjects(opponent);

		final Board board = scene.getBoard();

		final int [] myPPFC = getNumberOfPawnsPerFile(myObjects);
		final int [] hisPPFC = getNumberOfPawnsPerFile(hisObjects);

		// http://chessprogramming.wikispaces.com/Material
		// http://home.comcast.net/~danheisman/Articles/evaluation_of_material_imbalance.htm
		// larry kaufman values (for material)

		// 20130927: Sven Schule says use 10-20 for *pawns and 2-4 for mobility

		int value =
			countTotalObjectsValue(myObjects) - countTotalObjectsValue(hisObjects) // MAX 4092
			// need to verify what larry says about these
			// - 15 * (countNLeftBehindPawns(me, myObjects, hisMoves) - countNLeftBehindPawns(opponent, hisObjects, myMoves))
			- 15 * (countNIsolatedPawns(myPPFC) - countNIsolatedPawns(hisPPFC)) // MAX 15*8 = 120 // PPFC: pawns per file count
			- 15 * (countNDoublePawns(myPPFC) - countNDoublePawns(hisPPFC))	// MAX 15*4 = 60
			+ 3 * (calcMobility(scene, me) - calcMobility(scene, opponent)) // MAX 3*218 (according to http://chessprogramming.wikispaces.com/Chess+Position )
			;

		// maximum: 4092 + 120 + 60 + 3 * 218 = 4926

		return value;
	}

	int evalAndCheckPosition(final Scene scene, final PlayerColor side, final int currentSearchDistance) {
		if (scene.isCheckMate(side))
			return -(value_checkmate - currentSearchDistance);
		else if (scene.isStaleMate(side))
			return 0;
		else if (scene.isInsufficientMaterialDraw())
			return 0;

		return calcShannonValue(scene, side);
	}

	boolean checkMoveRepetition(final MoveRepetition history, final PlayerColor color, final long boardHash) {
		return history.contains(color, boardHash);
	}

	void emitProgress(final int depth, MoveResult objs) throws IOException {
		final long took = System.currentTimeMillis() - startedAt;

		if (took > 0) {
			double nodesPerSecond = sum(nNodesProcessed) * 1000.0 / took;

			io.progressCallbackNPS((int)nodesPerSecond);

			io.progressCallbackNNodes(sum(nNodesProcessed));

			io.progressCallbackTimeSearch((int)took);

			io.progressCallbackDebug("extensions: " + sum(nExtensions) + ", transposition table hits: " + sum(nTransTab) + ", cut offs: " + sum(nCutOffs));

			if (objs != null) {
				Move m = objs.getMove();
				int score = objs.getEval();

				if (m != null)
					io.progressCallbackBest(m, score);

				String pathStr = objs.getPv();

				if (pathStr == null)
					pathStr = "";

				io.progressCallbackXboardShowThinking(depth, score, took / 10, sum(nNodesProcessed), pathStr);
			}
		}
	}

	class TimeThread implements Runnable {
		long sleepTime;

		TimeThread(long sleepTime) {
			this.sleepTime = sleepTime;
		}

		public void run() {
			try {
				//Statics.log("TimeThread started");

				Thread.sleep(sleepTime);

				toDetected.set(true);
				Statics.log("TimeThread t/o triggered " + sleepTime);
			}
			catch(InterruptedException ie) {
				Statics.log("TimeThread exception: " + ie);
			}
		}
	}

	public void setupBookThreads(final List<Book> books) {
		bookThreads = new ArrayList<BookThread>();

		for(Book b : books)
			bookThreads.add(new BookThread(b));
	}

	public List<MoveResult> getOpeningBook(final Scene s, final Move lastMove, final PlayerColor pc) {
		try {
			Statics.log("Checking books for color " + pc);

			for(BookThread bt : bookThreads)
				bt.setupSearch(s, lastMove, pc);

			Statics.log("Book-threads notified, waiting for results");

			List<MoveResult> brs = new ArrayList<MoveResult>();

			for(BookThread bt : bookThreads) {
				MoveResult br = bt.getResult();

				if (br != null)
					brs.add(br);
			}

			Statics.log("Book-threads returned " + brs.size() + " moves");

			if (brs.isEmpty())
				return null;

			return brs;
		}
		catch(Exception e) {
			Statics.log("BOOK FAIL");
		}

		return null;
	}

	Object [] selectOpening(final List<MoveResult> brs) {
		int n = brs.size();

		if (n <= 2)
			return new Object [] { brs.get(r.nextInt(n)), new Long(-1) };

		Map<String, Object []> results = new HashMap<String, Object []>();

		for(MoveResult br : brs) {
			String brStr = br.getMove().toString();

			Object [] value = results.get(brStr);

			if (value == null)
				value = new Object[] { new Long(1), br };
			else {
				Long count = (Long)value[0];
				count++;
				value[0] = count;
			}

			results.put(brStr, value);
		}

		Long chosenCnt = new Long(-1);
		MoveResult chosen = null;
		for(String brStr : results.keySet()) {
			Object [] value = results.get(brStr);
			Long count = (Long)value[0];

			if (count > chosenCnt) {
				chosenCnt = count;
				chosen = (MoveResult)value[1];
			}
		}

		if (chosen == null)
			return null;

		return new Object [] { chosen, chosenCnt };
	}

	class BrainThread implements Runnable {
		private final Scene scene;
		private final int currentDepth;
		private final PlayerColor side;
		private final Move currentMove;
		private AlphaBeta result = null;
		private AlphaBeta ab;
		private final long boardHash;
		private final int counterSlotId, threadSlotId;
		private final AtomicBoolean childThreadAbortSearch;
		private Thread t;
		private Semaphore threadTerminatedFlag;
		private final AtomicBoolean hasTerminated = new AtomicBoolean(false);
		private MoveRepetition history;

		BrainThread(final Scene scene, final Move currentMove, final int currentDepth, final PlayerColor side, final AlphaBeta ab, final long boardHash, final int newCounterSlotId, final int newThreadSlotId, final AtomicBoolean childThreadAbortSearch, final Semaphore threadTerminatedFlag, final MoveRepetition history) {
			this.scene = scene;
			this.currentDepth = currentDepth;
			this.side = side;
			this.currentMove = currentMove;
			// literal copy as the caller already swapped+negated a & b
			// copy as AlphaBeta is not thread-safe
			this.ab = new AlphaBeta(ab);
			this.boardHash = boardHash;
			this.counterSlotId = newCounterSlotId;
			this.threadSlotId = newThreadSlotId;
			this.childThreadAbortSearch = childThreadAbortSearch;
			this.threadTerminatedFlag = threadTerminatedFlag;
			this.history = history;
		}

		public void setThread(final Thread t) {
			this.t = t;
		}

		public Thread getThread() {
			return t;
		}

		public AlphaBeta getSelectedMove() {
			return result;
		}

		public boolean getHasTerminated() {
			return hasTerminated.get();
		}

		public void run() {
			try {
				final Scene sceneAfterMove = getSceneAfterMove(scene, currentMove, side);

				final long newBoardHash = z.calc(boardHash, scene.getBoard(), sceneAfterMove, currentMove, side);

				result = search(sceneAfterMove, currentDepth, side, ab, true, newBoardHash, counterSlotId, threadSlotId, childThreadAbortSearch, null, history);

				if (result != null) // can happen due to cut-off/time-out
					result.setMove(currentMove);
			}
			catch(InterruptedException ie) {
			}
			catch(Exception e) {
				io.exception(e);
			}
			finally {
				hasTerminated.set(true);

				threadTerminatedFlag.release();
			}
		}
	}

	Scene getSceneAfterMove(final Scene scene, final Move currentMove, final PlayerColor side) {
		Scene sceneAfterMove = currentMove.getScene();

		if (sceneAfterMove == null)
			sceneAfterMove = scene.Move(currentMove);
		else
			currentMove.setScene(null); // help GC, link is not required

		return sceneAfterMove;
	}

	void endThreadAdminUpdate(final int threadSlotId) {
		threadCountLock[threadSlotId].acquireUninterruptibly();

		runningThreadsCount[threadSlotId]--;

		threadCountLock[threadSlotId].release();
	}

	int findMove(final List<Move> moves, final Move m) {
		final int nMoves = moves.size();

		for(int index=0; index<nMoves; index++) {
			if (m.equals(moves.get(index)))
				return index;
		}

		return -1;
	}

	void moveToFrontOfList(final List<Move> moves, final int shuffelIndex) {
		final Move shuffelElement = moves.get(shuffelIndex);

		moves.remove(shuffelIndex);

		moves.add(0, shuffelElement);
	}

	void joinThreads(final List<BrainThread> btList) throws InterruptedException {
		final int n = btList.size();

		for(int index=0; index<n; index++)
			btList.get(index).getThread().join();
	}

	final class ThreadsResults {
		boolean cutOff;
		int bestScore, nJoin;

		ThreadsResults(final boolean cutOff, final int bestScore, final int nJoin) {
			this.cutOff = cutOff;
			this.bestScore = bestScore;
			this.nJoin = nJoin;
		}

		boolean getCutOff() {
			return cutOff;
		}

		int getBestScore() {
			return bestScore;
		}

		int getNJoin() {
			return nJoin;
		}
	}

	ThreadsResults processThreadsResults(final List<BrainThread> btList, final AlphaBeta ab, final PlayerColor side, final AtomicBoolean abortSearch, final int counterSlot) throws InterruptedException {
		ThreadsResults out = null;
		int nJoin = 0, bestScore = -value_infinite;

		for(int index=btList.size() - 1; index >= 0; index--) {
			final BrainThread bt = btList.get(index);
			final Thread t = bt.getThread();

			if (bt.getHasTerminated()) {
				t.join();
				nJoin++;

				btList.remove(index);

				final AlphaBeta result = bt.getSelectedMove();

				if (result != null) {
					bestScore = Math.max(bestScore, result.getEval());

					if (ab.update(result)) {
						abortSearch.set(true);

						nCutOffs[counterSlot].incrementAndGet();

						out = new ThreadsResults(true, bestScore, nJoin);

						break;
					}
				}
			}
		}

		if (out == null)
			out = new ThreadsResults(false, bestScore, nJoin);

		return out;
	}

	// from http://chessprogramming.wikispaces.com/Quiescence+Search
	public AlphaBeta quiesceSearch(final Scene scene, final int currentDepth, final PlayerColor side, final AlphaBeta abIn, final AtomicBoolean abortSearch, final int counterSlotId) throws IOException, InterruptedException {
		final int currentSearchDistance = currentSearchDepth - currentDepth;

		int standPat = evalAndCheckPosition(scene, side, currentSearchDistance);

		int alpha = abIn.getAlpha();
		int beta = abIn.getBeta();

		if (standPat >= beta) // FIXME is this a cut-off? seems like it
			return new AlphaBeta(beta, true);

		if (alpha < standPat)
			alpha = standPat;

		scene.validateMoves(side);

		nQueiesceNodes[counterSlotId].incrementAndGet();

		nNodesProcessed[counterSlotId].incrementAndGet();

		final PlayerColor otherSide = Statics.opponentColor(side);

		final Board b = scene.getBoard();

		final List<Move> moves = scene.getMoveList(side);

		final MSCvictimType sortComparator = new MSCvictimType(b);
		Collections.sort(moves, sortComparator);

		int nextDepth = currentDepth - 1;

		final int nMoves = moves.size();

		for(int index=0; index<nMoves && !abortSearch.get() && !toDetected.get(); index++) {
			final Move currentMove = moves.get(index);

			// not a catching move and also not promoting to queen
			final int tx = currentMove.getTX(), ty = currentMove.getTY();

			if (b.getAt(tx, ty) != null || currentMove.getPromoteTo() == ChessObjectType.QUEEN) {
				Statics.log(currentMove.toString(b) + " " + Statics.toFENString(b, side, null, currentDepth, false));
				final Scene sceneAfterMove = getSceneAfterMove(scene, currentMove, otherSide);

				final AlphaBeta abForNextPly = new AlphaBeta(-beta, -alpha);

				final AlphaBeta result = quiesceSearch(sceneAfterMove, nextDepth, otherSide, abForNextPly, abortSearch, counterSlotId);
				int score = -result.getEval();

				if (score >= beta) // FIXME is this a cut-off? seems like it
					return new AlphaBeta(beta, true);

				if (score > alpha)
					alpha = score;
			}
		}

		return new AlphaBeta(alpha, false);
	}

	public AlphaBeta search(final Scene scene, final int currentDepth, final PlayerColor side, final AlphaBeta abIn, final boolean isThread, final long boardHash, final int counterSlotId, final int threadSlotId, final AtomicBoolean abortSearch, final Move prevItBestMove, final MoveRepetition history) throws IOException, InterruptedException {
		int alphaOrig = abIn.getAlpha();
		int bestScore = -value_infinite;

		final boolean topLevel = currentSearchDepth == currentDepth;

		if (toDetected.get()) {
			if (isThread)
				endThreadAdminUpdate(threadSlotId);

			return abIn.getResult();
		}

		final int currentSearchDistance = currentSearchDepth - currentDepth;

		if (currentDepth <= 0) {
			AlphaBeta rs = null;

			if (allowQueiescence) {
				rs = quiesceSearch(scene, currentDepth, side, abIn, abortSearch, counterSlotId);
			}
			else {
				scene.validateMoves(side);

				int score = evalAndCheckPosition(scene, side, currentSearchDistance);

				rs = new AlphaBeta(score, false);
			}

			if (isThread)
				endThreadAdminUpdate(threadSlotId);

			return rs;
		}

		nNodesProcessed[counterSlotId].incrementAndGet();

		if (!topLevel && checkMoveRepetition(history, side, boardHash) == true)
			return new AlphaBeta(0, false);

		scene.validateMoves(side);

		// generate move list
		final List<Move> moves = scene.getMoveList(side);
		int nMoves = moves.size();

		final PlayerColor otherSide = Statics.opponentColor(side);

		if (nMoves == 0) {
			// keep track of mates
			if (scene.isCheckMate(side)) { // to differentiate from stalemate
				mateAtLock.acquireUninterruptibly();

				if (currentSearchDistance < mateAtMin)
					mateAtMin = currentSearchDistance;
				else if (currentSearchDistance > mateAtMax)
					mateAtMax = currentSearchDistance;

				mateAtLock.release();
			}

			// evaluate
			int score = evalAndCheckPosition(scene, side, currentSearchDistance);

			if (isThread)
				endThreadAdminUpdate(threadSlotId);

			return new AlphaBeta(score, false);
		}

		// check transposition table if we've seen this "situation" before
		final TTElement ttObjs = ttEnabled ? transTab.get(boardHash) : null;
		//final TTElement ttObjs = null;

		Move ttMove = null;
		//if (false) 
		if (ttObjs != null) {
			nTransTab[counterSlotId].incrementAndGet();

			// see if the stored move was found at a greater distance than
			// the maximum distance we can search right now
			if (ttObjs.getDepth() >= currentDepth && (!topLevel || ttObjs.getMove() != null)) {
				Flag flag = ttObjs.getFlag();

				boolean useThis = false, isCutOff = false;

				int score = ttObjs.getEval(currentSearchDistance);

				if (flag == Flag.EXACT)
					useThis = true;
				else if (abIn.update(flag, score))
					isCutOff = useThis = true;

				if (useThis) {
					if (isThread)
						endThreadAdminUpdate(threadSlotId);

					return new AlphaBeta(ttObjs.getMove(), score, null, isCutOff);
				}
			}

			ttMove = ttObjs.getMove();
		}

		// order moves
		//if (topLevel)  // ... shallow shannon
		if (false) {
			for(int index=0; index<nMoves; index++) {
				Move curMove = moves.get(index);

				if (curMove.getScene() == null)
					curMove.setScene(scene.Move(curMove));
			}

			MSCshallowEval sortComparator = new MSCshallowEval(side);

			Collections.sort(moves, sortComparator);
		}
		else {	// ... by target-field-piece-value
			MSCvictimType sortComparator = new MSCvictimType(scene.getBoard());

			Collections.sort(moves, sortComparator);
		}

		// from what move-index to start threads
		// this way for both previous-best-move & tt-move, while those are calculated
		// now new threads are created
		int multiThreadIndex = 0;
		if (nMoves > 1) {
			// first check the "previously best move" and THEN the tt-move so
			// that the tt-move is always at the front
			if (prevItBestMove != null) {
				final int shuffelIndex = findMove(moves, prevItBestMove);

				// -1 would be really odd as the move searched for was the
				// best in the previous iteration. it may already be at
				// the front of the list though
				if (shuffelIndex > 0) {
					moveToFrontOfList(moves, shuffelIndex);
					multiThreadIndex++;
				}
			}
		}

		history.push(side, boardHash);

		final int nextDepth = currentDepth - 1;

		// need to "you need a reasonable "best score" to compare against, in 
		// order to guide the alpha-beta search" (Sven)
		if (multiThreadIndex < 1)
			multiThreadIndex = 1;
		// if the next ply is quiesce search, DO start threads as quiesce does
		// not fork and takes a while
		if (nextDepth == 0)
			multiThreadIndex = Math.max(0, multiThreadIndex - 1);

		// create list object for threads
		final List<BrainThread> btList = new ArrayList<BrainThread>();

		final long threadId = Thread.currentThread().getId();

		boolean cutOff = false;

		final AtomicBoolean childThreadAbortSearch = new AtomicBoolean(false);
		final Semaphore threadTerminatedFlag = new Semaphore(0, false);

		final int newThreadSlotId = (int)(threadId % numberOfThreadSlots);
		threadCountLock[newThreadSlotId].acquireUninterruptibly();
		int nToAdd = Math.min(nMoves - multiThreadIndex, maxThreadsPerSlot[newThreadSlotId] - runningThreadsCount[newThreadSlotId]);
		if (nToAdd < 0)
			nToAdd = 0;
		else
			runningThreadsCount[newThreadSlotId] += nToAdd;
		threadCountLock[newThreadSlotId].release();
		nThreadsCreated.addAndGet(nToAdd);

search_loop:
		for(int moveIndex=0; moveIndex < nMoves;) {
			if (abortSearch.get()) { // cut-off in an other thread / time-out
				childThreadAbortSearch.set(true); // "my" children can abort to

				// System.out.println("" + currentDepth + "> abort search");
				if (isThread)
					endThreadAdminUpdate(threadSlotId);

				history.pop(side);
				return null;
			}

			// end of search-time reached
			if (toDetected.get())
				break search_loop;

			// - to a try to get the lock, do not block on it: if an other thread is
			// working with this thread slot, it is most likely filled with work
			// already when it is finished so better start calculating instead of
			// sitting idle waiting for the lock
			final int newCounterSlotId = (int)(threadId % nNodesProcessed.length);

			if (nToAdd > 0 && moveIndex >= multiThreadIndex) {
				final AlphaBeta abForNextPly = new AlphaBeta(-abIn.getBeta(), -abIn.getAlpha());

				for(int newThreadNr=0; newThreadNr < nToAdd; newThreadNr++) {
					// can't move this to thread: java also doesn't like read-
					// access in multiple thread for ArrayDequeue type
					final MoveRepetition historyThread = new MoveRepetition(history);

					final BrainThread bt = new BrainThread(scene, moves.get(moveIndex), nextDepth, otherSide, abForNextPly, boardHash, newCounterSlotId, newThreadSlotId, childThreadAbortSearch, threadTerminatedFlag, historyThread);

					final Thread t = new Thread(bt, "" + nextDepth);
					t.start();

					bt.setThread(t);
					btList.add(bt);

					moveIndex++;
				}

				nToAdd = 0;
			}

			assert(!cutOff);

			if (moveIndex < nMoves) {
				final Move currentMove = moves.get(moveIndex++);

				final Scene sceneAfterMove = getSceneAfterMove(scene, currentMove, otherSide);

				final long newBoardHash = z.calc(boardHash, scene.getBoard(), sceneAfterMove, currentMove, otherSide);

				// long dummyZ = z.calc(sceneAfterMove.getBoard(), otherSide);
				// if (dummyZ != newBoardHash)
				// System.out.println("FAIL");

				// do not pick when it is a repeating move unless it is the only
				// possible one
				final AlphaBeta abForNextPly = new AlphaBeta(-abIn.getBeta(), -abIn.getAlpha());

				AlphaBeta objs = search(sceneAfterMove, nextDepth, otherSide, abForNextPly, false, newBoardHash, counterSlotId, threadSlotId, childThreadAbortSearch, null, history);

				// end of search-time reached
				if (toDetected.get())
					break search_loop;

				if (objs != null) {
					objs.setMove(currentMove);

					bestScore = Math.max(bestScore, -objs.getEval());

					cutOff = abIn.update(objs);

					if (cutOff) {
						childThreadAbortSearch.set(true);

						nCutOffs[counterSlotId].incrementAndGet();

						break search_loop;
					}
				}
			}

			if (threadTerminatedFlag.tryAcquire()) {
				ThreadsResults result = processThreadsResults(btList, abIn, side, childThreadAbortSearch, counterSlotId);
				cutOff = result.getCutOff();

				// score has already been negated in processThreadsResults()
				bestScore = Math.max(bestScore, -result.getBestScore());

				int nJoin = result.getNJoin();
				if (nJoin > 1) // tryAcquire also acquired one
					threadTerminatedFlag.acquire(nJoin - 1);

				if (cutOff)
					break search_loop;
			}
		}

		// we decrease the count here as the following loop will not (hardly) use cpu
		if (isThread)
			endThreadAdminUpdate(threadSlotId);

		// thread results
		if (!cutOff && !btList.isEmpty() && !toDetected.get()) {
			for(;;) {
				ThreadsResults result = processThreadsResults(btList, abIn, side, childThreadAbortSearch, counterSlotId);

				int nJoin = result.getNJoin();
				if (nJoin > 1) // tryAcquire also acquired one
					threadTerminatedFlag.acquire(nJoin - 1);

				bestScore = Math.max(bestScore, -result.getBestScore());

				if (result.getCutOff() || btList.isEmpty())
					break;

				threadTerminatedFlag.acquire();
			}
		}

		// runs when either cut-offs or t/o
		if (!btList.isEmpty())
			joinThreads(btList);

		// do not add to transtab if time-out as the search was not complete
		Move m = abIn.getMove();

		if (!toDetected.get() && ttEnabled && abIn.getAlpha() > alphaOrig /*&& (ttMove == null || m == null || ttMove.equals(m) == false)*/) {
			Flag flag = null;

			if (bestScore <= alphaOrig)
				flag = Flag.UPPER;
			else if (bestScore >= abIn.getBeta())
				flag = Flag.LOWER;
			else
				flag = Flag.EXACT;

			transTab.put(boardHash, m, bestScore, (byte)currentDepth, flag, currentSearchDistance);
		}

		if (!topLevel)
			scene.destructor();

		history.pop(side);

		return abIn;
	}

	void stopPonder() {
	}

	void asyncStopSearch() {
		if (!stopAll.get())
			Statics.log("Was not running!");

		stopAll.set(true);
		toDetected.set(true);
	}

	public MoveResult calculateMove(final int moveNr, final Scene scene, final int endDepth, final PlayerColor myColor, final MoveRepetition history, final long maxDuration, final int maxExtension, final Move lastMove, final boolean isPonder) throws IOException, InterruptedException {
		startedAt = System.currentTimeMillis();

		stopAll.set(false);

		Statics.log("FEN: " + Statics.toFENString(scene.getBoard(), myColor, null, moveNr, false));

		// intialize 'scene'
		scene.createMoveList(myColor);
		scene.validateMoves(myColor);

		if (!isPonder) {
			List<MoveResult> obResults = getOpeningBook(scene, lastMove, myColor);
			if (obResults != null) {
				Object [] obResultTemp = selectOpening(obResults);

				if (obResultTemp != null) {
					MoveResult obResult = (MoveResult)obResultTemp[0];

					io.progressCallback("Book: " + obResult + " out of " + obResults.size() + " with " + (Long)obResultTemp[1] + " hits");
					io.progressCallbackXboardShowThinking(obResult.getDepth(), obResult.getEval(), (System.currentTimeMillis() - startedAt) / 10, 0, obResult.getMove() + " " + obResult.getPv());

					// FIXME convert pv to list-of-moves
					return obResult;
				}
			}
		}

		List<Move> moveList = scene.getMoveList(myColor);
		if (moveList.size() == 1)
			return new MoveResult(calcShannonValue(scene, myColor), 1, moveList.get(0), "CALC");

		// time management
		finishBefore = maxDuration > 0 ? startedAt + maxDuration: -1;

		toDetected.set(false);

		Thread timeThreadThread = null;
		TimeThread timeThread = null;
		Statics.log("calculateMove: maxDuration=" + maxDuration);
		if (maxDuration > 0) {
			timeThread = new TimeThread(maxDuration > cleanupTimeFrame ? maxDuration - cleanupTimeFrame : maxDuration);

			timeThreadThread = new Thread(timeThread);
			timeThreadThread.setPriority(Thread.MAX_PRIORITY);
			timeThreadThread.start();
		}

		this.moveNr = moveNr;

		// statistics
		setupCounters();

		RunStats rs = new RunStats();
		Thread rst = new Thread(rs, "RunStats");
		rst.start();

		// check transposition table if we've seen this "situation" before
		final long boardHash = z.calc(scene.getBoard(), myColor);

		this.maxExtension = -maxExtension;
		io.progressCallback("Will think " + (maxDuration / 1000.0) + " seconds, max depth: " + endDepth + ", max extensions: " + maxExtension + ", # threads: " + currentMaxNProcs + ", threads per slot: " + threadsPerSlot);

		MoveResult result = null;

		int alpha = -value_infinite, beta = value_infinite;

		// http://www.talkchess.com/forum/viewtopic.php?topic_view=threads&p=528463&t=48855&sid=fa70310d9024e6cd922c2538efe1fd7b
		// "a basic aspiration algorithm"
		// A simple scheme could be something like that: 
		// (0) search the wull window at depth 1 and get score(1). 
		// (1) set alpha=score(1)-100cp and beta=score(1)+100cp, and search depth=2 with that window: 
		// (i) if score is within the bounds: all good, proceed to the next iteration (set depth=3 and go to (1)). 
		// (ii) if score <= alpha, then set alpha=-inf and retry (go back to (1)) 
		// (iii) if score >= beta, then set beta=+inf and retry (go back to (1)) 

		Move prevBest = null;
		int prevMateMinAtDepth = Integer.MAX_VALUE, prevMateMaxAtDepth = Integer.MIN_VALUE;
		int id = 1;
		for(;;) {
			if (endDepth != 0 && id > endDepth)
				break;

			currentSearchDepth = id;

			mateAtMin = Integer.MAX_VALUE;
			mateAtMax = Integer.MIN_VALUE;

			if (toDetected.get()) {
				io.progressCallback("Search aborted due to t/o at depth " + id);
				break;
			}

			io.progressCallbackDepthReached(id);
			io.progressCallbackDebug("alpha: " + alpha + ", beta: " + beta);

			for(int index=0; index<numberOfThreadSlots; index++)
				runningThreadsCount[index] = 0;

			long threadId = Thread.currentThread().getId();

			int counterSlotId = (int)(threadId % nNodesProcessed.length);
			int threadSlotId = (int)(threadId % numberOfThreadSlots);

			runningThreadsCount[threadSlotId] = 1; // current "thread"!

			io.progressCallbackWindowSize(alpha, beta);

			AlphaBeta searchAb = new AlphaBeta(alpha, beta);
			AlphaBeta objsAb = search(scene, id, myColor, searchAb, true, boardHash, counterSlotId, threadSlotId, stopAll, prevBest, history);
			MoveResult objs = objsAb != null ? objsAb.getMoveResult(id, "CALC") : null;

			if (mateAtMin != Integer.MAX_VALUE)
				io.progressCallbackMateAt(mateAtMin);

			if (mateAtMin != prevMateMinAtDepth || mateAtMax != prevMateMaxAtDepth) {
				io.progressCallback("Mate at depth min: " + mateAtMin + ", max: " + mateAtMax);

				prevMateMinAtDepth = mateAtMin;
				prevMateMaxAtDepth = mateAtMax;
			}

			if (objs != null) {
				Move currentMove = objs.getMove();
				int currentScore = objs.getEval();

				io.progressCallback("search returned: " + objs);

				if (result == null && currentMove != null)
					result = objs;

				if (Math.abs(currentScore) == 10000) { /* TEMP HACK FIXME */
					result = objs;
					id++;
					prevBest = currentMove;
				}
				else if (currentScore <= alpha) {
					io.progressCallback("score " + currentScore + " less than alpha " + alpha);
					alpha = -value_infinite;
				}
				else if (currentScore >= beta) {
					io.progressCallback("score " + currentScore + " more than beta " + beta);
					beta = value_infinite;
				}
				else {
					alpha = Math.max(-value_infinite, currentScore - aspirationWindowSize);
					beta = Math.min(value_infinite, currentScore + aspirationWindowSize);
					io.progressCallback("retry with window " + alpha + "..." + beta);
					result = objs;
					id++;

					prevBest = currentMove;
				}
			}
			else {
				io.progressCallback("No result returned, last chosen: " + result);

				if (alpha == -value_infinite && beta == value_infinite) {
					id++;
				}
				else {
					alpha = -value_infinite;
					beta = value_infinite;
				}
			}

			emitProgress(currentSearchDepth, objs);
		}

		double took = (System.currentTimeMillis() - startedAt) / 1000.0;

		if (finishBefore > 0)
			io.progressCallback("seconds left: " + (finishBefore - System.currentTimeMillis()) / 1000.0);
		else
			io.progressCallbackDebug("took: " + took);

		rst.interrupt();

		if (timeThreadThread != null)
			timeThreadThread.interrupt();

		// this can happen if a search is done with depth 1 and a too big depth/maxExtension
		if (result == null || result.getMove() == null) {
			Statics.log("No move chosen - emergency selection");
			result = new MoveResult(calcShannonValue(scene, myColor), 1, scene.getMoveList(myColor).get(0), "CALC");
		}

		// result should always != null
		// it can only be null when time-out happens during search
		// and no lower depth was searched before which did finish
		// in time (iterative deepening)
		if (result != null)
			io.progressBenchmark(endDepth, currentMaxNProcs, threadsPerSlot, sum(nNodesProcessed), took, scene.getBoard(), result, sum(nExtensions), sum(nCutOffs), sum(nTransTab));

		if (gcThread != null)
			gcThread.poll();

		stopAll.set(true);

		return result;
	}
}
