package com.vanheusden.pfa;

import java.util.*;

class ChessObjectKnight extends ChessObject {
	public boolean equals(Object o) {
		final ChessObject oco = (ChessObject)o;

		return oco instanceof ChessObjectKnight && oco.getX() == x && oco.getY() == y && oco.getColor() == color && oco.getNMoves() == nMoves;
	}

	public ChessObjectKnight(final int x, final int y, final PlayerColor color) {
		super(x, y, color, 0);
	}

	public ChessObjectKnight(final int x, final int y, final PlayerColor color, final int nMoves) {
		super(x, y, color, nMoves);
	}

	public ChessObject duplicate() {
		return new ChessObjectKnight(x, y, color, nMoves);
	}

	public ChessObjectType getType() {
		return ChessObjectType.KNIGHT;
	}

	public String getTypeCharacter(final boolean uppercase) {
		final String character = ChessObjectType.KNIGHT.getName();

		if (uppercase)
			return character.toUpperCase();

		return character;
	}

	public void getMoves(final Scene scene, final List<Move> moves, final ChessObject king, final Move lastMove) {
		assert board == null;
		assert this.moves == null;

		board = scene.getBoard();
		this.moves = moves;

		final int xp1 = x + 1, yp1 = y + 1;
		final int xp2 = x + 2, yp2 = y + 2;
		final int xm1 = x - 1, ym1 = y - 1;
		final int xm2 = x - 2, ym2 = y - 2;
		final boolean yp1_valid = yp1 <= 7;
		final boolean yp2_valid = yp2 <= 7;
		final boolean ym1_valid = ym1 >= 0;
		final boolean ym2_valid = ym2 >= 0;

		if (xp1 <= 7)
		{
			if (yp2_valid)
				conditionalAddMove(x, y, xp1, yp2);
			if (ym2_valid)
				conditionalAddMove(x, y, xp1, ym2);
		}
		if (xm1 >= 0)
		{
			if (yp2_valid)
				conditionalAddMove(x, y, xm1, yp2);
			if (ym2_valid)
				conditionalAddMove(x, y, xm1, ym2);
		}

		if (xp2 <= 7)
		{
			if (yp1_valid)
				conditionalAddMove(x, y, xp2, yp1);
			if (ym1_valid)
				conditionalAddMove(x, y, xp2, ym1);
		}

		if (xm2 >= 0)
		{
			if (yp1_valid)
				conditionalAddMove(x, y, xm2, yp1);
			if (ym1_valid)
				conditionalAddMove(x, y, xm2, ym1);
		}

		assert canDoNMoves <= 8;

		generatedListOfMoves = true;
	}
}
