package com.vanheusden.pfa;

import java.util.*;

class ChessObjectRook extends ChessObject {
	public boolean equals(Object o) {
		final ChessObject oco = (ChessObject)o;

		return oco instanceof ChessObjectRook && oco.getX() == x && oco.getY() == y && oco.getColor() == color && oco.getNMoves() == nMoves;
	}

	private void addMoveCastling(final int fx, final int fy, final int tx) {
		moves.add(new Move(fx, fy, tx));

		canDoNMoves++;
	}

	/* public stuff */
	public ChessObjectRook(final int x, final int y, final PlayerColor color) {
		super(x, y, color, 0);
	}

	public ChessObjectRook(final int x, final int y, final PlayerColor color, final int nMoves) {
		super(x, y, color, nMoves);
	}

	public ChessObject duplicate() {
		return new ChessObjectRook(x, y, color, nMoves);
	}

	public ChessObjectType getType() {
		return ChessObjectType.ROOK;
	}

	public String getTypeCharacter(final boolean uppercase) {
		final String character = ChessObjectType.ROOK.getName();

		if (uppercase)
			return character.toUpperCase();

		return character;
	}

	public void getMoves(final Scene scene, final List<Move> moves, final ChessObject king, final Move lastMove) {
		assert board == null;
		assert this.moves == null;

		this.moves = moves;
		board = scene.getBoard();

		getMovesPath( 0,  1);
		getMovesPath( 1,  0);
		getMovesPath( 0, -1);
		getMovesPath(-1,  0);

		/* CASTLING */
		if (isFirstMove() && king.isFirstMove()) {
			assert y == 0 || y == 7;

			///////////////////////////////////////////////////////
			// IN VALIDATEMOVES FIELD 3,Y / 5,Y MUST BE CHECKED! //
			///////////////////////////////////////////////////////

			/* short/long castling? */
			if (x == 0) {	/* long */
				if (board.getAt(1, y) == null && board.getAt(2, y) == null && board.getAt(3, y) == null) {
					addMoveCastling(4, y, 2);
				}
			}
			else {	/* short */
				if (board.getAt(5, y) == null && board.getAt(6, y) == null) {
					addMoveCastling(4, y, 6);
				}
			}
		}

		assert canDoNMoves <= 18;

		generatedListOfMoves = true;
	}
}
