package com.vanheusden.pfa;

import java.io.*;
import java.util.*;
import java.util.concurrent.*;

class IO_ConsoleANSI extends IO_Console {
	double previousScore = 0;
	final int moveWidth = 6;
	final int xMoveWhite = 4, xMoveBlack = xMoveWhite + moveWidth + 1;
	final int xPosBoard = xMoveBlack + 7;
	final int xDataCol1 = xPosBoard + 10;
	final int dataWidth = 13;
	final int xDataCol2 = xDataCol1 + dataWidth;
	final int xDataCol3 = xDataCol2 + dataWidth;
	final int xDataCol4 = xDataCol3 + dataWidth;
	final int yInput = 10;
	final int yMoveEnterResult = yInput + 1;
	final int yLogSeperator = yMoveEnterResult + 1;
	final int yCopyright = 24;
	final int yLog = yLogSeperator + 1, nLogLines = yCopyright - yLog;
	final int termColorWhite = 7, termColorCyan = 6, termColorMagenta = 5, termColorBlue = 4;
	final int termColorBrown = 3, termColorGreen = 2, termColorRed = 1, termColorBlack = 0;
	String [] logWindow = new String[nLogLines];
	long nRunnableAvg = 0, nRunnableAvgCnt = 0;
	Semaphore outputLock = new Semaphore(1, false);

	String abbreviate(long value) {
		final long peta = 1000000000000000l;
		if (value >= peta)
			return String.format("%.1fP", value / (double)peta);

		final long tera = 1000000000000l;
		if (value >= tera)
			return String.format("%.1fT", value / (double)tera);

		final long giga = 1000000000;
		if (value >= giga)
			return String.format("%.1fG", value / (double)giga);

		final long mega = 1000000;
		if (value >= mega)
			return String.format("%.1fM", value / (double)mega);

		final long kilo = 1000;
		if (value >= kilo)
			return String.format("%.1fk", value / (double)kilo);

		return String.format("%3d", value);
	}

	public void center(String msg) {
		int len = msg.length();
		int x = Math.max(0, 40 - len / 2);

		System.out.print("\r");

		for(int nr=0; nr<x; nr++)
			System.out.print(" ");

		System.out.println(msg);
	}

	void lock() {
		outputLock.acquireUninterruptibly();
	}

	void unlock() {
		outputLock.release();
	}

	public void progressCallback(String what) {
		Statics.log(what);

		lock();
		addLogLine(what);
		unlock();
	}

	public void progressCallbackDebug(String what) {
		if (Statics.verbose > 0) {
			Statics.log(what);

			lock();
			addLogLine(what);
			unlock();
		}
	}

	void put(int x, int y, String header, String what, int len) {
		term_move_cursor(x, y);

		for(int space=0; space<len; space++)
			System.out.print(" ");

		term_move_cursor(x, y);

		term_set_fg_color(termColorRed);

		System.out.print(header + " ");

		term_set_fg_color(termColorCyan);

		if (what.length() > len)
			System.out.print(what.substring(0, len));
		else
			System.out.print(what);

		term_set_fg_color(termColorWhite);
	}

	public void progressCallbackXboardShowThinking(int id, int score, long durationCentiSeconds, long nodesVisited, String details) throws IOException {
		String what = "" + id + " " + score + " " + durationCentiSeconds + " " + nodesVisited + " " + details;

		Statics.log(what);

		lock();
		putSmiley(xDataCol1, 7, score, previousScore);

		put(xDataCol2, 1, "seen:", abbreviate(nodesVisited), dataWidth);

		put(xDataCol1, 2, "path:", String.format("%s", details), 79 - (xDataCol1 + 6));

		put(xDataCol2, 3, "score:", String.format("%5d", score), dataWidth);
		unlock();
	}

	public void progressCallbackNPS(int nps) throws IOException {
		lock();
		put(xDataCol3, 1, "n/s:", abbreviate(nps), dataWidth);
		unlock();
	}

	public void progressCallbackTimeSearch(int ms) throws IOException {
		Statics.log("Time searched: " + ms + "ms");

		lock();
		put(xDataCol4, 1, "ms", abbreviate(ms), dataWidth);
		unlock();
	}

	public void progressCallbackBest(Move move, int score) throws IOException {
		Statics.log("Best move: " + move + " (" + score + ")");

		lock();
		put(xDataCol1, 3, "best:", "" + move, dataWidth);

		putSmiley(xDataCol1, 7, score, previousScore);
		unlock();
	}

	public void progressCallbackNNodes(long nNodes) throws IOException {
		Statics.log("Total nodes searched: " + nNodes);

		lock();
		put(xDataCol2, 1, "seen:", abbreviate(nNodes), dataWidth);
		unlock();
	}

	public void progressCallbackDepthReached(int depth) throws IOException {
		Statics.log("Depth reached: " + depth + " (thread: " + Thread.currentThread().getName() + ")");

		lock();
		put(xDataCol3, 3, "depth:", "" + depth, dataWidth);
		unlock();
	}

	public void progressCallbackCurrMove(Move move) throws IOException {
		Statics.log("Current move: " + move);

		lock();
		put(xDataCol4, 3, "cur:", "" + move, dataWidth);
		unlock();
	}

	public void progressThreadStats(int cntRunnable, int cntBlocked, int slotsUsage, int slotsInUse, long nThreadsCreated) {
		lock();
		put(xDataCol1, 4, "RUN:", String.format("%3d", cntRunnable), dataWidth);
		put(xDataCol2, 4, "BLK:", String.format("%3d", cntBlocked), dataWidth);
		put(xDataCol3, 4, "STU:", String.format("%3d", slotsUsage), dataWidth);
		put(xDataCol4, 4, "STI:", String.format("%3d", slotsInUse), dataWidth);
		put(xDataCol1, 5, "THC:", abbreviate(nThreadsCreated), dataWidth);

		nRunnableAvg += cntRunnable;
		nRunnableAvgCnt++;
		put(xDataCol3, 7, "RA:",  String.format("%.1f", nRunnableAvg / (double)nRunnableAvgCnt), dataWidth);
		unlock();
	}

	public void processChessStats(long nExtensions, long nTransTab, long nCutOffs, long nQueiesceNodes) {
		lock();
		put(xDataCol2, 5, "EXT:", abbreviate(nExtensions), dataWidth);
		put(xDataCol3, 5, "TT:" , abbreviate(nTransTab), dataWidth);
		put(xDataCol4, 5, "CUT:", abbreviate(nCutOffs), dataWidth);
		put(xDataCol1, 6, "QN:",  abbreviate(nQueiesceNodes), dataWidth);
		unlock();
	}

	public void progressTTLockDistribution(double sd) {
		lock();
		put(xDataCol2, 6, "TLD:",  String.format("%6.2f", sd), dataWidth);
		unlock();
	}

	public void progressCallbackWindowSize(int alpha, int beta) {
		lock();
		put(xDataCol3, 6, "alpha:",  String.format("%5d", alpha), dataWidth);
		put(xDataCol4, 6, "beta:",  String.format("%5d", beta), dataWidth);
		unlock();
	}

	public void progressCallbackMateAt(int distance) {
		lock();
		put(xDataCol2, 7, "mate:",  String.format("%3d", distance), dataWidth);
		unlock();
	}

	ReturnData getMove(Scene scene, PlayerColor currentPlayer) throws IOException {
		Move move;
		int c = 0;

		do {
			BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

			do {
				StringBuffer error = new StringBuffer("");
				String moveString;

				do {
					lock();
					term_move_cursor(1, yInput);
					term_clear_line();

					System.out.print("Your move: ");
					unlock();

					moveString = in.readLine().trim().toLowerCase();

					if (moveString.equals("quit"))
						return new ReturnData(ReturnType.RET_QUIT, null);
					if (moveString.equals("undo"))
						return new ReturnData(ReturnType.RET_UNDO, null);
				}
				while(moveString.length() < 2);

				lock();
				term_move_cursor(1, yMoveEnterResult);
				term_clear_line();
				unlock();

				move = Statics.SANToMove(moveString, scene.getBoard(), currentPlayer, scene.getMoveList(currentPlayer), error);

				if (move == null) {
					lock();
					addLogLine("SAN (" + moveString + "): " + error);
					unlock();
					error = new StringBuffer("");

					move = Statics.LANToMove(moveString, scene.getBoard(), currentPlayer, scene.getMoveList(currentPlayer), error);
				}

				if (move == null)
					addLogLine("LAN (" + moveString + "): " + error);
			}
			while(move == null);

			lock();
			term_move_cursor(1, yMoveEnterResult);
			term_clear_line();
			unlock();

			boolean fail = false;
			do {
				lock();
				term_move_cursor(1, yInput);
				term_clear_line();

				System.out.print("Move " + Statics.moveToString(move, false) + "? ");

				if (fail) {
					term_reverse();
					System.out.print("(Y/N)");
					term_reset_attributes();
					term_set_fg_color(termColorWhite);
				}
				else {
					System.out.print("(y/n)");
					fail = true;
				}
				unlock();

				c = Character.toUpperCase((char)in.read());
			}
			while(c != 'Y' && c != 'N');
		}
		while(c != 'Y');

		lock();
		term_move_cursor(1, yInput);
		term_clear_line();
		System.out.print("Your move: " + move);
		unlock();

		return new ReturnData(ReturnType.RET_MOVE, move);
	}

	public IO_ConsoleANSI(String pgnFile, int maxDepth, int maxExtension, boolean random, double maxDuration, String boardInit, final Brain brain, final boolean ponder, List<String> beepers) throws Exception {
		super(pgnFile, maxDepth, maxExtension, random, maxDuration, boardInit, brain, ponder, beepers);

		if (boardInit != null) {
			Object [] epd = Statics.EPDToScene(boardInit);
			scene = (Scene)epd[0];
			currentPlayer = (PlayerColor)epd[1];
		}
		else {
			scene = new Scene();
		}

		this.pgnFile = pgnFile;
	}

	void displayBoard(Board b, Move lastMove) {
		for(int y=0; y<8; y++) {
			int ty = 7 - y;

			term_reset_attributes();

			term_move_cursor(xPosBoard, y + 1);
			term_set_fg_color(termColorBrown);
			System.out.print("" + (ty + 1));
			term_set_fg_color(termColorWhite);

			for(int x=0; x<8; x++) {
				ChessObject obj = b.getAt(x, ty);

				term_move_cursor(xPosBoard + 1 + x, y + 1);

				term_reset_attributes();
				term_faint();

				if (lastMove != null && lastMove.getMoveType() == MoveType.REGULAR &&
					((lastMove.getFX() == x && lastMove.getFY() == y) ||
					(lastMove.getTX() == x && lastMove.getTY() == y))) {
					term_blink_rapid();
				}

				if (obj == null) {
					if (((x + y) & 1) == 1)
						term_reverse();

					System.out.print(" ");
					continue;
				}

				term_bright();

				if (obj.getColor() == PlayerColor.BLACK)
					term_reverse();

				if (obj.getColor() == PlayerColor.BLACK)
					System.out.print(obj.getType().getName().toUpperCase());
				else
					System.out.print(obj.getType().getName());

				term_reset_attributes();
				term_set_fg_color(termColorWhite);
			}
		}

		term_move_cursor(xPosBoard + 1, 9);
		term_set_fg_color(termColorBrown);
		System.out.print("abcdefgh");
		term_set_fg_color(termColorWhite);

	}

	void term_set_fg_color(int c) {
		System.out.print("\033[" + (30 + c) + "m");
	}

	void term_bright() {
		System.out.print("\033[1m");
	}

	void term_faint() {
		System.out.print("\033[2m");
	}

	void term_blink_slow() {
		System.out.print("\033[5m");
	}

	void term_blink_rapid() {
		System.out.print("\033[6m");
	}

	void term_reverse() {
		System.out.print("\033[7m");
	}

	void term_reset_attributes() {
		System.out.print("\033[0m");
	}

	void term_cls() {
		System.out.print("\033[2J");
	}

	void term_move_cursor(int x, int y) {
		System.out.print("\033[" + y + ";" + x + "H");
	}

	void term_clear_line() {
		System.out.print("\033[2K");
	}

	void showLogWindow() {
		term_set_fg_color(termColorMagenta);

		for(int y=0; y<logWindow.length; y++) {
			term_move_cursor(1, yLog + y);
			term_clear_line();

			if (logWindow[y] == null)
				continue;

			if (logWindow[y].length() > 79)
				System.out.print(logWindow[y].substring(0, 79));
			else
				System.out.print(logWindow[y]);
		}

		term_set_fg_color(termColorWhite);
	}

	void addLogLine(String line) {
		for(int i=1; i<logWindow.length; i++)
			logWindow[i - 1] = logWindow[i];

		logWindow[logWindow.length - 1] = line;

		showLogWindow();
	}

	void term_move_below_screen() {
		term_move_cursor(1, 24);
		System.out.println("\033[99B");
	}

	public void putSmiley(int x, int y, double score, double prevValue) {
		if (score - prevValue < 0.001d)
			put(x, y, "conf:", ":-(", dataWidth);
		else if (score - prevValue > 0.001d)
			put(x, y, "conf:", ":-)", dataWidth);
		else
			put(x, y, "conf:", ":-|", dataWidth);
	}

	void showMoveList() {
		for(int y=0; y<8; y++) {
			term_move_cursor(1, y);
			put(xMoveWhite, y, "", "", moveWidth);
			put(xMoveBlack, y, "", "", moveWidth);
		}

		int y = 1;
		for(int i=Math.max(0, h.size() - 8); i<h.size(); i++) {
			term_move_cursor(1, y);
			System.out.print(String.format("%2d ", i + 1));

			put(xMoveWhite, y, "", h.get(i).getWhiteMove().toString(), moveWidth);

			if (h.get(i).getBlackMove() != null)
				put(xMoveBlack, y, "", h.get(i).getBlackMove().toString(), moveWidth);
			else
				put(xMoveBlack, y, "", "", moveWidth);

			y++;
		}
	}

	void putCopyright() {
		term_move_cursor(1, yCopyright);

		String copyrightMsg = Version.getName() + " " + Version.getVersion() + "/" + Version.getRevision() + " http://vanheusden.com/DBP/";
		int copyrightLen = copyrightMsg.length();
		int copyrightX = Math.max(79 - copyrightLen, 0);

		term_reset_attributes();

		term_set_fg_color(termColorGreen);

		for(int x=0; x<copyrightX; x++)
			System.out.print(" ");

		System.out.print(copyrightMsg);

		term_set_fg_color(termColorWhite);
	}

	void term_store_screen() {
		System.out.println("\033[?1049h,");
	}

	void term_restore_screen() {
		System.out.println("\033[?1049l,");
	}

	void initTerminal() {
		term_cls();
		term_reset_attributes();
		term_set_fg_color(termColorWhite);
		term_move_cursor(1, 1);

		term_move_cursor(1, yLogSeperator);
		for(int x=0; x<80; x++)
			System.out.print("-");

		putCopyright();
	}

	boolean checkCheckMate(PlayerColor color) throws IOException {
		if (scene.isCheckMate(color)) {
			term_move_below_screen();

			put_logo();

			String msg = "" + color + " is check-mate";
			Statics.log(msg);
			center(msg);

			if (pgnObject != null) {
				if (color == PlayerColor.WHITE)
					pgnObject.closePgnFile("0-1");
				else
					pgnObject.closePgnFile("1-0");
			}

			return true;
		}

		return false;
	}

	boolean checkStaleMate(PlayerColor color) throws IOException {
		if (scene.isStaleMate(color)) {
			term_move_below_screen();

			put_logo();

			String msg = "" + color + " is stale-mate";
			Statics.log(msg);
			center(msg);

			if (pgnObject != null) {
				if (color == PlayerColor.WHITE)
					pgnObject.closePgnFile("0-1");
				else
					pgnObject.closePgnFile("1-0");
			}

			return true;
		}

		return false;
	}

	boolean checkInsufficientMaterialDraw() throws IOException {
		if (scene.isInsufficientMaterialDraw()) {
			term_move_below_screen();

			put_logo();

			String msg = "insufficient material draw";
			Statics.log(msg);
			center(msg);

			if (pgnObject != null)
				pgnObject.closePgnFile("1/2-1/2");

			return true;
		}

		return false;
	}

	public void loop() throws IOException, InterruptedException {
		int it = 0;
		PlayerColor humanColor = PlayerColor.WHITE;
		int c;

		term_store_screen();

		initTerminal();

		Statics.log(Statics.startupLine);

		boolean fail = false;
		do {
			lock();
			term_move_cursor(1, 1);
			term_clear_line();
			System.out.print("Play with black or white? ");

			if (fail) {
				term_reverse();
				System.out.print("(W/B) ");
				term_reset_attributes();
			}
			else {
				System.out.print("(w/b) ");
				fail = true;
			}
			unlock();

			c = Character.toUpperCase(System.in.read());
		}
		while(c != 'W' && c != 'B');

		lock();
		if (c == 'B') {
			humanColor = PlayerColor.BLACK;
			Statics.log("Opponent plays with black");
			pgnObject = new Pgn(PlayerColor.WHITE, pgnFile);
		}
		else {
			Statics.log("Opponent plays with white");
			pgnObject = new Pgn(PlayerColor.BLACK, pgnFile);
		}

		addLogLine("DeepBrutePos plays with " + Statics.opponentColor(humanColor));
		unlock();

		Move lastMove = null;
		HistoryElement he = null;

		initTerminal();

undo_loop:
		for(;;) {
			lock();
			term_reset_attributes();
			term_set_fg_color(termColorWhite);
			term_move_cursor(1, 1);

			if (currentPlayer == PlayerColor.WHITE) {
				it++;

				he = new HistoryElement();

				term_move_cursor(xDataCol1, 1);
				term_set_fg_color(termColorRed);
				System.out.print("move " + it);
			}
			unlock();

			scene.validateMoves(currentPlayer);

			if (checkCheckMate(PlayerColor.WHITE) || checkCheckMate(PlayerColor.BLACK) ||
				checkStaleMate(PlayerColor.WHITE) || checkStaleMate(PlayerColor.BLACK) ||
				checkInsufficientMaterialDraw()) {
				break;
			}

			lock();
			displayBoard(scene.getBoard(), lastMove);
			unlock();

			MoveResult move = null;

			String fen = Statics.toFENString(scene.getBoard(), currentPlayer, lastMove, it, false);

			if (currentPlayer == humanColor) {
				ReturnData ret = getMove(scene, currentPlayer);

				if (ret.getReturnType() == ReturnType.RET_QUIT) {
					lock();
					term_restore_screen();
					put_logo();
					unlock();
					break;
				}

				if (ret.getReturnType() == ReturnType.RET_UNDO && h.size() >= 1) {
                                        HistoryElement curHe = h.remove(h.size() - 1);

					if (currentPlayer == PlayerColor.WHITE) {
						Object [] epd = Statics.EPDToScene(curHe.getWhiteBeforeFen());
						scene = (Scene)epd[0];
					}
					else {
						Object [] epd = Statics.EPDToScene(curHe.getBlackBeforeFen());
						scene = (Scene)epd[0];
					}

					lock();
					showMoveList();
					unlock();

					continue undo_loop;
				}

				lastMove = ret.getMove();

				move = new MoveResult(0, 0, lastMove, "opponent");

				Statics.log("" + it + ". opponent moves: " + Statics.moveToString(lastMove, false));
			}
			else {
				lock();
				term_move_cursor(1, yInput);
				term_clear_line();

				term_set_fg_color(termColorBlue);
				term_blink_slow();
				System.out.print(" * DBP is calculating * ");
				term_reset_attributes();
				term_set_fg_color(termColorWhite);
				unlock();

				move = brain.calculateMove(it, scene, maxDepth, currentPlayer, moveRepHist, (long)(maxDuration * 1000.0), maxExtension, lastMove, false);

				String out = Statics.moveToString(move.getMove(), false) + " (" + move.getEval() + ")";

				Statics.log("" + it + ". POS moves: " + out);
			}

			if (currentPlayer == PlayerColor.WHITE) {
				he.setWhite(fen, move.getMove());
				h.add(he);
			}
			else {
				he.setBlack(fen, move.getMove());

				he = new HistoryElement();
			}

			lock();
			showMoveList();
			unlock();

			Statics.log(move.toString());

			pgnObject.pgnEmitMove(scene.getBoard(), move.getMove(), currentPlayer);

			moveRepHist.push(currentPlayer, brain.getHash(scene.getBoard(), currentPlayer));

			scene = scene.Move(move.getMove());

			double score = move.getEval();
			lock();
			putSmiley(xDataCol1, 7, score, previousScore);
			unlock();
			previousScore = score;

			currentPlayer = Statics.opponentColor(currentPlayer);
		}
	}

	void put_logo() {
		byte [] logo_ans = new byte [] {
			0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
				0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37,
				0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x3a, 0x20, 0x20, 0x20, 0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b,
				0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20, 0x20, 0x20,
				0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x1b, 0x5b,
				0x30, 0x6d, 0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x2e, 0x20, 0x2e,
				0x20, 0x2e, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b,
				0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x31,
				0x3b, 0x34, 0x31, 0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b,
				0x33, 0x31, 0x3b, 0x34, 0x37, 0x6d, 0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30,
				0x3b, 0x35, 0x3b, 0x33, 0x33, 0x3b, 0x34, 0x30, 0x6d, 0x74, 0x1b, 0x5b,
				0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x37, 0x6d, 0x40, 0x1b,
				0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x40,
				0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x20, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x38, 0x3a, 0x20, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x2e,
				0x20, 0x20, 0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b, 0x30, 0x3b,
				0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20, 0x20, 0x2e,
				0x20, 0x20, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x33,
				0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x33, 0x35, 0x3b,
				0x34, 0x31, 0x6d, 0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x31,
				0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x31, 0x3b, 0x34, 0x31, 0x6d, 0x40, 0x58, 0x38, 0x38, 0x38, 0x38, 0x1b,
				0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x38,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d,
				0x40, 0x38, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b,
				0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x2e, 0x2e, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x20,
				0x20, 0x20, 0x2e, 0x20, 0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x53,
				0x74, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x31,
				0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33,
				0x35, 0x3b, 0x34, 0x30, 0x6d, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x35, 0x3b, 0x34, 0x31, 0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b,
				0x33, 0x33, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20, 0x74, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x40,
				0x3b, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x35, 0x3b, 0x34,
				0x31, 0x6d, 0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x33, 0x33, 0x3b, 0x34, 0x31,
				0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34,
				0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x31, 0x3b,
				0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x38, 0x38, 0x20, 0x20, 0x20, 0x2e,
				0x20, 0x20, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x2e,
				0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x25, 0x25, 0x25, 0x3a, 0x20, 0x3b, 0x25, 0x20, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x53, 0x25, 0x25,
				0x53, 0x74, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x20, 0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x35, 0x3b, 0x34, 0x30, 0x6d, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x35,
				0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x25, 0x58, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x2e,
				0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x74, 0x20, 0x20, 0x20, 0x20, 0x2e, 0x20, 0x1b, 0x5b, 0x30, 0x6d,
				0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x20, 0x20, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x58, 0x1b, 0x5b, 0x30, 0x3b, 0x35,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x2e, 0x38, 0x1b, 0x5b,
				0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x25,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37,
				0x6d, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30,
				0x3b, 0x34, 0x37, 0x6d, 0x74, 0x25, 0x25, 0x25, 0x25, 0x74, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x40, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d,
				0x38, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x20, 0x20, 0x20, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x20,
				0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20, 0x2e, 0x3a, 0x1b, 0x5b,
				0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x53, 0x74,
				0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37,
				0x6d, 0x3b, 0x25, 0x74, 0x74, 0x25, 0x25, 0x74, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x58, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x3b, 0x74, 0x74,
				0x74, 0x25, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x31, 0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b,
				0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x20, 0x20, 0x2e, 0x20,
				0x20, 0x20, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x58,
				0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x74, 0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b,
				0x34, 0x37, 0x6d, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x33,
				0x3b, 0x34, 0x30, 0x6d, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33,
				0x35, 0x3b, 0x34, 0x31, 0x6d, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x3b, 0x25, 0x25, 0x74, 0x25,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x20, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34,
				0x37, 0x6d, 0x74, 0x25, 0x74, 0x74, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x2e, 0x1b, 0x5b, 0x30, 0x3b,
				0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x3b, 0x1b, 0x5b, 0x30,
				0x3b, 0x35, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x40, 0x40, 0x1b,
				0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20,
				0x20, 0x20, 0x20, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x6d,
				0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x20, 0x3b, 0x53, 0x3b, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b,
				0x35, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x53, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x25, 0x25,
				0x74, 0x74, 0x25, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x20, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b,
				0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x74, 0x74, 0x25, 0x74, 0x74, 0x74,
				0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37,
				0x6d, 0x58, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34,
				0x31, 0x6d, 0x38, 0x40, 0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33,
				0x33, 0x3b, 0x34, 0x30, 0x6d, 0x2e, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x35,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x2e, 0x20, 0x20, 0x20, 0x20,
				0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20, 0x2e, 0x20, 0x3a, 0x1b,
				0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x33, 0x3b, 0x34, 0x31, 0x6d, 0x53,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d,
				0x38, 0x58, 0x25, 0x74, 0x25, 0x74, 0x74, 0x25, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x40, 0x3a, 0x1b, 0x5b,
				0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x74,
				0x74, 0x25, 0x74, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b,
				0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x74,
				0x20, 0x20, 0x2e, 0x20, 0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20,
				0x20, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30,
				0x3b, 0x34, 0x37, 0x6d, 0x25, 0x25, 0x74, 0x74, 0x74, 0x25, 0x74, 0x1b,
				0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38,
				0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37,
				0x6d, 0x74, 0x74, 0x74, 0x25, 0x74, 0x74, 0x74, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x58, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x58, 0x38,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x40, 0x3b, 0x20, 0x20, 0x20, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x6d,
				0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x20, 0x20, 0x20, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x3b, 0x74, 0x74, 0x25, 0x74,
				0x25, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x58, 0x2e, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x30, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x74, 0x25, 0x74, 0x74, 0x25, 0x74,
				0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x31,
				0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34,
				0x31, 0x6d, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b,
				0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31,
				0x3b, 0x34, 0x37, 0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x33, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x2e, 0x20, 0x2e, 0x20, 0x20,
				0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x2e, 0x20, 0x20, 0x3b, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x74,
				0x25, 0x74, 0x25, 0x74, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x74, 0x20, 0x38, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x74, 0x74, 0x74, 0x74,
				0x25, 0x74, 0x74, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x58, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x31, 0x3b, 0x34, 0x31, 0x6d, 0x40, 0x58, 0x1b, 0x5b, 0x30, 0x3b, 0x35,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20, 0x20, 0x2e, 0x20,
				0x38, 0x53, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20,
				0x20, 0x2e, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b,
				0x34, 0x37, 0x6d, 0x25, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x3b, 0x1b, 0x5b, 0x30,
				0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x40, 0x3a, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x3a,
				0x74, 0x25, 0x74, 0x25, 0x74, 0x74, 0x25, 0x74, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x58, 0x25, 0x38,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d,
				0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x37,
				0x6d, 0x38, 0x38, 0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x38, 0x20, 0x2e, 0x20, 0x1b, 0x5b, 0x30, 0x6d,
				0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x20, 0x20, 0x20, 0x2e, 0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x74, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20,
				0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b,
				0x34, 0x37, 0x6d, 0x74, 0x74, 0x74, 0x25, 0x74, 0x74, 0x25, 0x74, 0x74,
				0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31,
				0x6d, 0x38, 0x74, 0x58, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31,
				0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33,
				0x35, 0x3b, 0x34, 0x31, 0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x35,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x53, 0x20, 0x20, 0x20, 0x20,
				0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x2e, 0x20, 0x20, 0x20, 0x2e,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x40, 0x3a, 0x40, 0x20, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b,
				0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x74, 0x74, 0x25, 0x74, 0x74, 0x74,
				0x25, 0x74, 0x74, 0x25, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x31, 0x3b, 0x34, 0x31, 0x6d, 0x38, 0x74, 0x25, 0x25, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x33, 0x6d, 0x38, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x38,
				0x53, 0x2e, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20,
				0x20, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x20, 0x20, 0x2e, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x2e, 0x1b, 0x5b,
				0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x74,
				0x74, 0x25, 0x74, 0x25, 0x74, 0x74, 0x25, 0x74, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x40, 0x74, 0x74,
				0x58, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x31, 0x3b, 0x34,
				0x31, 0x6d, 0x38, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31,
				0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x20, 0x1b, 0x5b, 0x30, 0x6d,
				0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x20, 0x20, 0x2e, 0x20, 0x20,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37,
				0x6d, 0x74, 0x74, 0x25, 0x74, 0x74, 0x74, 0x74, 0x25, 0x74, 0x74, 0x74,
				0x3a, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34,
				0x31, 0x6d, 0x38, 0x74, 0x74, 0x74, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x35, 0x3b, 0x34, 0x37, 0x6d, 0x53, 0x1b, 0x5b, 0x30,
				0x3b, 0x35, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x3a, 0x1b, 0x5b,
				0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x58, 0x1b,
				0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20,
				0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20, 0x20, 0x20, 0x2e, 0x20,
				0x20, 0x20, 0x20, 0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30,
				0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x74, 0x74, 0x74, 0x25, 0x74, 0x25, 0x74,
				0x74, 0x25, 0x74, 0x74, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x58, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x33, 0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b,
				0x35, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x38, 0x1b, 0x5b, 0x30,
				0x3b, 0x35, 0x3b, 0x33, 0x35, 0x3b, 0x34, 0x30, 0x6d, 0x20, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x35, 0x3b, 0x34, 0x31, 0x6d, 0x25, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x31, 0x3b, 0x34, 0x31, 0x6d, 0x38,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20,
				0x2e, 0x20, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x40, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x74, 0x38, 0x25, 0x3a,
				0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x74, 0x3b, 0x3a, 0x1b, 0x5b, 0x30,
				0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x53, 0x20, 0x20,
				0x20, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x31,
				0x3b, 0x34, 0x31, 0x6d, 0x40, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20, 0x74, 0x1b, 0x5b, 0x30, 0x6d,
				0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x20, 0x20, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x20, 0x2e, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x58,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d,
				0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x3a, 0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x74, 0x58, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x2e, 0x2e, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x1b, 0x5b, 0x30,
				0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x53, 0x3a, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x74,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d,
				0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x2e, 0x20, 0x20, 0x20, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30,
				0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x2e, 0x20, 0x20,
				0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x20,
				0x20, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b,
				0x34, 0x37, 0x6d, 0x3b, 0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b,
				0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x1b, 0x5b, 0x30, 0x3b, 0x35,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x25, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x3a, 0x3b,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d,
				0x2e, 0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x3a, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b,
				0x34, 0x37, 0x6d, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
				0x38, 0x3b, 0x20, 0x2e, 0x1b, 0x5b, 0x30, 0x6d, 0x0a, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x20,
				0x20, 0x20, 0x20, 0x2e, 0x20, 0x2e, 0x20, 0x1b, 0x5b, 0x30, 0x3b, 0x31,
				0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x3b, 0x1b, 0x5b, 0x30, 0x3b,
				0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x2e, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x58, 0x1b, 0x5b,
				0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x2e,
				0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d,
				0x2e, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x74, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x74, 0x53, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37,
				0x3b, 0x34, 0x37, 0x6d, 0x2e, 0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b,
				0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20, 0x40, 0x38, 0x1b, 0x5b, 0x30,
				0x3b, 0x31, 0x3b, 0x33, 0x30, 0x3b, 0x34, 0x37, 0x6d, 0x74, 0x1b, 0x5b,
				0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38, 0x1b,
				0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x20,
				0x20, 0x20, 0x20, 0x2e, 0x3a, 0x20, 0x20, 0x20, 0x1b, 0x5b, 0x30, 0x6d,
				0x0a, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x20, 0x20, 0x2e, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x38,
				0x38, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37,
				0x6d, 0x20, 0x20, 0x20, 0x3b, 0x1b, 0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33,
				0x37, 0x3b, 0x34, 0x37, 0x6d, 0x40, 0x74, 0x25, 0x38, 0x1b, 0x5b, 0x30,
				0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x2e, 0x3a, 0x1b,
				0x5b, 0x30, 0x3b, 0x31, 0x3b, 0x33, 0x37, 0x3b, 0x34, 0x37, 0x6d, 0x58,
				0x74, 0x58, 0x1b, 0x5b, 0x30, 0x3b, 0x35, 0x3b, 0x33, 0x37, 0x3b, 0x34,
				0x37, 0x6d, 0x3a, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x2e, 0x20, 0x20,
				0x1b, 0x5b, 0x30, 0x6d
		};

		String str = new String(logo_ans);

		System.out.println(str + " DeepBrutePos - http://vanheusden.com/DBP/");
	}
}
